/*******************************************************************************
    Filename:     encode.js

    Created:      23 August 2006 (15:15:08)
    Created by:   Jan Schreiber (a.k.a. MrSpock)
    Last Updated: 9 July 2011

    Comments:
    This scripts adds several HTML/JavaScript specific encoding
    and decoding mechanisms to PSPad.
    Furthermore, it includes an rgb(d,d,d)<>#HHHHHH converter.

    If you want to contact me, decode the following line: ;o)
    %6A%61%6E%2Eschreiber%40%75%6E%69%2D%64%75%65%2E%64%65
    Or send a PM to user MrSpock via the PSPad web site.

*******************************************************************************/

// Constants:
module_name = "js_encode";
module_ver = "1.2";
menu_name = "HTML Encoding";
prot_msg = "File is read-only!";

function Init()
{
	addMenuItem("Escape TXT (all chars)", menu_name, "Escape", "");
	addMenuItem("Encode URI (standard)", menu_name, "myEncodeURI", "");
	addMenuItem("Unescape URI by utf-8", menu_name, "unescapeURI", "");
	addMenuItem("Unencode URI by utf-8", menu_name, "undecodeURI", "");
	addMenuItem("-", menu_name, "", "");
	addMenuItem("Encoded mailto-link", menu_name, "Escape_Mail", "");
	addMenuItem("Escape HTML (All Characters)", menu_name, "Escape_HTML", "");
	addMenuItem("Escape HTML (Only Special Characters)", menu_name, "Escape_HTML_Special", "");
	addMenuItem("Unescape HTML", menu_name, "unescape_HTML", "");
	addMenuItem("-", menu_name, "", "");
	addMenuItem("rgb2hex", menu_name, "rgb2hex", "");
	addMenuItem("hex2rgb", menu_name, "hex2rgb", "");
	addMenuItem("-", menu_name, "", "");
	addMenuItem("dec2Hex", menu_name, "conv2hex", "");
	addMenuItem("hex2Dec", menu_name, "conv2dec", "");
	addMenuItem("-", menu_name, "", "");
	addMenuItem("Edit script", menu_name, "openScript", "");
}

function Escape()
// Encodes text selected in the editor into the %HEX format used
// in URLs. Works on every character.
{
	var obj1 = newEditor();
	try
	{
		obj1.assignActiveEditor();
	}
	catch(e)
	{
		return;
	}
	if (obj1.readOnly())
	{
		echo(prot_msg);
		return;
	}
	var inpString = obj1.SelText();
	var resString = "";
	var n, curCharNum;
 
	for (n=0; n<inpString.length; n++)
	{
		curCharNum = inpString.charCodeAt(n);
		resString += "%"+dec2Hex(curCharNum);
	}
	obj1.SelText(resString);
}

function Escape_Mail()
// Transforms an e-mail address selected in the text or entered via the 
// input box into a mailto link that looks like
// <a href="mailto:%6A%6F%68%6E%40%64%6F%65%2E%63%6F%6D">
// &#106;&#111;&#104;&#110;&#64;&#100;&#111;&#101;&#46;&#99;&#111;&#109;</a>
// and is hopefully invisible for harvesters (a simple anti-spam measure). 
{
	var obj1 = newEditor();
	try
	{
		obj1.assignActiveEditor();
	}
	catch(e)
	{
		return;
	}
	if (obj1.readOnly())
	{
		echo(prot_msg);
		return;
	}
	var defaultAddress = "john.doe@example.org";
	var inpString = obj1.SelText();
	var resString = "<a href=\"mailto:";
	var n, curCharNum;
	var re = new RegExp("^[-_.0-9a-z]+@[-_.0-9a-z]+[.][a-z]{1,4}$");
	if (!inpString.match(re))
	{
		if (!inpString === "")
		{
			defaultAddress = inpString;
		}
		inpString = inputText("Please enter a valid e-mail address:", defaultAddress);
		if (inpString === "")
		{
			return;
		}
	}
 
	for (n=0; n<inpString.length; n++)
	{
		curCharNum = inpString.charCodeAt(n);
		resString += "%"+dec2Hex(curCharNum);
	}
	resString += "\">";
	for (n=0; n<inpString.length; n++)
	{
		curCharNum = inpString.charCodeAt(n);
		resString += "&#"+curCharNum+";";
	}
	resString += "</a>";
	obj1.SelText(resString);
}

function Escape_HTML()
// Escapes selected text to HTML charcodes (&#123;)
{
	var obj1 = newEditor();
	try
	{
		obj1.assignActiveEditor();
	}
	catch(e)
	{
		return;
	}
	if (obj1.readOnly())
	{
		echo(prot_msg);
		return;
	}
	var inpString = obj1.SelText();
	var resString = "";
	var n, curCharNum;

	for (n=0; n<inpString.length; n++)
	{
		curCharNum = inpString.charCodeAt(n);
		resString += "&#"+curCharNum+";";
	}
	obj1.SelText(resString);
}

function Escape_HTML_Special()
// Escapes special characters in selected text to HTML charcodes,
// such as &amp;, &quot;, &gt;, &lt;
{
	var obj1 = newEditor();
	try
	{
		obj1.assignActiveEditor();
	}
	catch(e)
	{
		return;
	}
	if (obj1.readOnly())
	{
		echo(prot_msg);
		return;
	}
	var inpString = obj1.SelText();
	var resString = "";
	var n, curChar;

	for (n=0; n<inpString.length; n++)
	{
		curChar = inpString.charAt(n);
		switch (curChar)
		{
			case '<':
				resString += "&lt;"; break;
			case '>':
				resString += "&gt;"; break;
			case '&':
				resString += "&amp;";  break;
			case '"':
				resString += "&quot;"; break;
			case '\'':
				resString += "&apos;"; break;
			case '':
				resString += "&reg;";  break;
			case '':
				resString += "&copy;"; break;
			default:
				resString += curChar; break;
		}
	}
	obj1.SelText(resString);
}

function unescape_HTML()
// Unescapes HTML number codes (&#123;) to text.
// (a little messy, needs work)
{
	var obj1 = newEditor();
	try
	{
		obj1.assignActiveEditor();
	}
	catch(e)
	{
		return;
	}
	if (obj1.readOnly())
	{
		echo(prot_msg);
		return;
	}
	var inpString = obj1.SelText();
	var resString = "";
	var n = 0;
	var curChar, strToDec, tmpString;
	var re = new RegExp("^(&#(\\d)+;)+$", "g");

	while (!inpString.match(re))
	{
		inpString = inputText("Please enter a sequence of one or\nmore strings of the form &&#n;",inpString);
		if (inpString === "")
			{ return; }
	}

	while (n<inpString.length)
	{
		strToDec = "";
		i = 0;
		do
		{
			curChar = inpString.charAt(n+i);
			strToDec += curChar;
			i++;
		}
		while (curChar != ";");
		n = n+i;
		tmpString = strToDec.replace("&#","");
		tmpString = tmpString.replace(";","");
		tmpString = dec2Hex(tmpString);
		tmpString = unescape("%"+tmpString);
		resString += tmpString;
	}
	obj1.SelText(resString);
}

function myEncodeURI()
// Encodes text selected in the editor into the %HEX format used
// in URLs. Works on special characters only.
{
	var obj1 = newEditor();
	try
	{
		obj1.assignActiveEditor();
	}
	catch(e)
	{
		return;
	}
	if (obj1.readOnly())
	{
		echo(prot_msg);
		return;
	}
	var inpString = obj1.SelText();
	obj1.SelText(encodeURI(inpString));
}

function unescapeURI()
// Reverses Escape() and escapeURI().
{
	var obj1 = newEditor();
	try
	{
		obj1.assignActiveEditor();
	}
	catch(e)
	{
		return;
	}
	if (obj1.readOnly())
	{
		echo(prot_msg);
		return;
	}
	var inpString = obj1.SelText();
	obj1.SelText(unescape(inpString));
}

function undecodeURI()
{
	var obj1 = newEditor();
	try
	{
		obj1.assignActiveEditor();
	}
	catch(e)
	{
		return;
	}
	if (obj1.readOnly())
	{
		echo(prot_msg);
		return;
	}
	var inpString = obj1.SelText();
	try
	{
		obj1.SelText(decodeURI(inpString));
	}
	catch(e)
	{
		echo ("Charset Error ...");
		return;
	}
}

function rgb2hex()
// Transforms css color values given in rgb(1,2,3) form to #XXXXXX form.
// Should automatically find the strings to replace in the selected text.
{
	var editor = newEditor();
	try
	{
		editor.assignActiveEditor();
	}
	catch(e)
	{
		return;
	}
	if (editor.readOnly())
	{
		echo(prot_msg);
		return;
	}
	var inpString = editor.SelText();
	var outpString = "";
	var re = new RegExp("rgb\\((\\d{1,3})(\\s)*,(\\s)*(\\d{1,3})(\\s)*,(\\s)*(\\d{1,3})\\)", "gi");
	outpString = inpString.replace( re, function (str,p1,p2,p3,p4,p5,p6,p7,offset,s) { return "#"+dec2Hex(p1)+dec2Hex(p4)+dec2Hex(p7); } );
	editor.SelText(outpString);
}

function hex2rgb()
// Opposite of rgb2hex()
{
	var editor = newEditor();
	try
	{
		editor.assignActiveEditor();
	}
	catch(e)
	{
		return;
	}
	if (editor.readOnly())
	{
		echo(prot_msg);
		return;
	}
	var inpString = editor.SelText();
	var outpString = "";
	var re = new RegExp("#([a-f0-9]{2})([a-f0-9]{2})([a-f0-9]{2})", "gi");
	outpString = inpString.replace
		( re,
			function (str,p1,p2,p3,offset,s) //p1-p3: parenthesized submatches
			{
				return "rgb("+hex2Dec(p1)+","+hex2Dec(p2)+","+hex2Dec(p3)+")";
			}
		);
	editor.SelText(outpString);
}

function conv2hex()
{
	var obj1 = newEditor();
	try
	{
		obj1.assignActiveEditor();
	}
	catch(e)
	{
		return;
	}
	if (obj1.readOnly())
	{
		echo(prot_msg);
		return;
	}
	var inpString = obj1.SelText();
	var outpString = 0;
	var re = new RegExp("^[0-9]+$", "g");
	while (!inpString.match(re))
	{
		inpString = InputText("Please enter a decimal number:");
		if (inpString === "") return;
	}
	outpString = dec2Hex(inpString);
	if (outpString == "Out of Range")
	{
		echo(outpString);
	}
	else
	{
		obj1.SelText(outpString);
	}
}

function conv2dec()
{
	var obj1 = newEditor();
	try
	{
		obj1.assignActiveEditor();
	}
	catch(e)
	{
		return;
	}
	if (obj1.readOnly())
	{
		echo(prot_msg);
		return;
	}
	var inpString = obj1.SelText();
	var re = new RegExp("^(0x)?([0-9a-f])+$", "gi");
	while (!inpString.match(re))
	{
		inpString = InputText("Enter a hexadecimal number:");
		if (inpString === "") return;
	}
	obj1.SelText(parseInt(inpString, 16));
}

function dec2Hex(dec)
{
	dec = parseInt(dec, 10);
	if (!isNaN(dec))
	{
		hexChars = "0123456789ABCDEF";
		if (dec > 255)
		{
			return "Out of Range";
		}
		var i = dec % 16;
		var j = (dec - i) / 16;
		var result = "";
		result += hexChars.charAt(j) + hexChars.charAt(i);
		return result;
	}
	else
	{
		return NaN;
	}
}

function hex2Dec(hex)
{
	var result = parseInt(hex, 16);
	return result;
}

function openScript()
{
	var obj1 = newEditor();
	try
	{
		obj1.openFile(moduleFileName(module_name));
	}
	catch(e)
	{
		echo("Error opening file!");
	}
}
